//***************************************************************
//  Towers.java         Authors: Lewis and Loftus
//                      Updated: Bary W Pollack
//
//  Solves the classic Towers of Hanoi puzzle.
//***************************************************************

public class Towers {
    //------------------------------------------------------------
    //  Creates a TowersOfHanoi puzzle and solves it.
    //------------------------------------------------------------
    public static void main(String[] args) {
        System.out.println("Solving the Towers of Hanoi Puzzle");
        System.out.println();
        for (int i = 1; i < 6; i++) {
            TowersOfHanoi towers = new TowersOfHanoi(i);
            towers.solve();
        }
    }
}

class TowersOfHanoi {

    //------------------------------------------------------------
    //  Sets up the puzzle with the specified number of disks.
    //------------------------------------------------------------
    public TowersOfHanoi(int disks) {
        totalDisks = disks;
    }

    //------------------------------------------------------------
    //  Performs the initial call to moveTower to solve the puzzle.
    //------------------------------------------------------------
    public void solve() {
        System.out.println("Solution for " + totalDisks + " disks:");
        moveTower(totalDisks, 1, 3, 2);
        System.out.println();
    }
   
    //------------------------------------------------------------
    //  Moves the specified number of disks from one tower to
    //  another by moving a subtower of n-1 disks out of the way,
    //  moving one disk, then moving the subtower back.
    //  Base case of 1 disk.
    //------------------------------------------------------------
    private void moveTower(int numDisks, int start, int end, int temp) {
        if (numDisks <= 1)
            moveOneDisk(start, end);
        else {
            moveTower(numDisks-1, start, temp, end);
            moveOneDisk(start, end);
            moveTower(numDisks-1, temp, end, start);
        }
    }

    //------------------------------------------------------------
    //  Prints instructions to move one disk from the specified
    //  start tower to the specified end tower.
    //------------------------------------------------------------
    private void moveOneDisk(int start, int end) {
        System.out.println("Move one disk from " + start + " to " + end);
    }

    private int totalDisks; 
}

/* ..... Output from the program .....

Solving the Towers of Hanoi Puzzle

Solution for 1 disks:
Move one disk from 1 to 3

Solution for 2 disks:
Move one disk from 1 to 2
Move one disk from 1 to 3
Move one disk from 2 to 3

Solution for 3 disks:
Move one disk from 1 to 3
Move one disk from 1 to 2
Move one disk from 3 to 2
Move one disk from 1 to 3
Move one disk from 2 to 1
Move one disk from 2 to 3
Move one disk from 1 to 3

Solution for 4 disks:
Move one disk from 1 to 2
Move one disk from 1 to 3
Move one disk from 2 to 3
Move one disk from 1 to 2
Move one disk from 3 to 1
Move one disk from 3 to 2
Move one disk from 1 to 2
Move one disk from 1 to 3
Move one disk from 2 to 3
Move one disk from 2 to 1
Move one disk from 3 to 1
Move one disk from 2 to 3
Move one disk from 1 to 2
Move one disk from 1 to 3
Move one disk from 2 to 3

Solution for 5 disks:
Move one disk from 1 to 3
Move one disk from 1 to 2
Move one disk from 3 to 2
Move one disk from 1 to 3
Move one disk from 2 to 1
Move one disk from 2 to 3
Move one disk from 1 to 3
Move one disk from 1 to 2
Move one disk from 3 to 2
Move one disk from 3 to 1
Move one disk from 2 to 1
Move one disk from 3 to 2
Move one disk from 1 to 3
Move one disk from 1 to 2
Move one disk from 3 to 2
Move one disk from 1 to 3
Move one disk from 2 to 1
Move one disk from 2 to 3
Move one disk from 1 to 3
Move one disk from 2 to 1
Move one disk from 3 to 2
Move one disk from 3 to 1
Move one disk from 2 to 1
Move one disk from 2 to 3
Move one disk from 1 to 3
Move one disk from 1 to 2
Move one disk from 3 to 2
Move one disk from 1 to 3
Move one disk from 2 to 1
Move one disk from 2 to 3
Move one disk from 1 to 3

..... End of Output ..... */

